package digital.laboratory.platform.sys.controller;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.sys.entity.Area;
import digital.laboratory.platform.common.security.annotation.Inner;
import digital.laboratory.platform.sys.service.AreaService;
import digital.laboratory.platform.sys.vo.AreaVO;
import digital.laboratory.platform.common.core.constant.CacheConstants;
import digital.laboratory.platform.common.core.util.R;
import io.swagger.annotations.Api;
import lombok.RequiredArgsConstructor;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 省/地/县 3级行政区划 前端控制器
 * </p>
 *
 * @author Zhang Xiaolong
 * created at 2022-03-10 09:45:58
 * @describe 省/地/县 3级行政区划前端控制器
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/comm/area")
@Api( tags = "01-行政区划管理")
public class CommAreaController {
    private final AreaService areaService;

    /**
     * 通过 id 查询对象
     * @param id 对象的 id
     * @return 对象信息
     */
    @GetMapping("/{id}")
    public R getById(@PathVariable String id) {
        return R.ok(areaService.getById(id));
    }

    /**
     * 通过 id 查询对象
     * @param id 对象的 id
     * @return 对象信息
     */
    @GetMapping("/inner/{id}")
    @Inner
    public R innerGetById(@PathVariable String id) {
        return R.ok(areaService.getById(id));
    }

    /**
     * 分页查询对象列表
     * @param page 分页对象
     * @return 分页对象
     */
    @GetMapping("/page")
    public R<IPage> getPage(Page page, Area area) {
        page.setOrders(OrderItem.ascs("id", "name"));

        return R.ok(areaService.page(page, Wrappers.<Area>query()
                .like(area.getId()!=null,"id", area.getId())
                .like(area.getName()!=null,"name", area.getName())
        ));
    }

    /**
     * 取省列表
     * @return 同类型字典
     */
    @GetMapping("/provinces")
    @Cacheable(value = CacheConstants.AREA_PROVINCES_DETAILS)
    public R<List<Area>> getProvinces() {
        return R.ok(areaService.list(Wrappers.<Area>query()
                .like("id", "__0000")
                .orderByAsc("sort")
        ));
    }

    /**
     * 取省列表
     * @return 同类型字典
     */
    @GetMapping("/inner/provinces")
    @Inner
    @Cacheable(value = CacheConstants.AREA_PROVINCES_DETAILS)
    public R<List<Area>> innerGetProvinces() {
        return R.ok(areaService.list(Wrappers.<Area>query()
                .like("id", "__0000")
                .orderByAsc("sort")
        ));
    }

    /**
     * 取指定省的地市列表
     * @param provinceId 省
     * @return 地市列表
     */
    @GetMapping("/cities/{provinceId}")
	@Cacheable(value = CacheConstants.AREA_CITIES_DETAILS, key = "#provinceId")
    public R<List<Area>> getCitys(@PathVariable String provinceId) {
        if (provinceId.length()<2) {
            return R.failed("省代码不能小于2位");
        }
        provinceId = provinceId.substring(0, 2);
        return R.ok(areaService.list(Wrappers.<Area>query()
                .like("id", provinceId +"__00")
                .ne("id", provinceId +"0000")
        ));
    }

    /**
     * 取指定省的地市列表
     * @param provinceId 省
     * @return 地市列表
     */
    @GetMapping("/inner/cities/{provinceId}")
    @Inner
    @Cacheable(value = CacheConstants.AREA_CITIES_DETAILS, key = "#provinceId")
    public R<List<Area>> innerGetCitys(@PathVariable String provinceId) {
        if (provinceId.length()<2) {
            return R.failed("省代码不能小于2位");
        }
        provinceId = provinceId.substring(0, 2);
        return R.ok(areaService.list(Wrappers.<Area>query()
                .like("id", provinceId +"__00")
                .ne("id", provinceId +"0000")
        ));
    }

    /**
     * 取指定地市的区县列表
     * @param cityId 地市
     * @return 区县列表
     */
    @GetMapping("/districts/{cityId}")
	@Cacheable(value = CacheConstants.AREA_DISTRICTS_DETAILS, key = "#cityId")
    public R<List<Area>> getDistricts(@PathVariable String cityId) {
        if (cityId.length()<4) {
            return R.failed("地市代码不能小于4位");
        }
        cityId = cityId.substring(0, 4);
        return R.ok(areaService.list(Wrappers.<Area>query()
                .like("id", cityId +"__")
                .ne("id", cityId +"00")
        ));
    }

    /**
     * 取指定地市的区县列表
     * @param cityId 地市
     * @return 区县列表
     */
    @GetMapping("/inner/districts/{cityId}")
    @Inner
    @Cacheable(value = CacheConstants.AREA_DISTRICTS_DETAILS, key = "#cityId")
    public R<List<Area>> innerGetDistricts(@PathVariable String cityId) {
        if (cityId.length()<4) {
            return R.failed("地市代码不能小于4位");
        }
        cityId = cityId.substring(0, 4);
        return R.ok(areaService.list(Wrappers.<Area>query()
                .like("id", cityId +"__")
                .ne("id", cityId +"00")
        ));
    }

//    /**
//     * 取指定区域的下属子区域
//     * @param dispCode 地市
//     * @return 下一级区域列表
//     */
//    @GetMapping("/sub_area")
////    @Cacheable(value = CacheConstants.AREA_SUB_DETAILS, key = "#code")
//    public R<List<AreaVO>> getSubAreas(String dispCode) {
//        if (StrUtil.isBlank(dispCode) || "000000".equals(dispCode)) {
//            // 没有提供 code 或 code 是 000000, 表示要取全部省
//            return R.ok(areaService.listAreaVO(Wrappers.<Area>query()
//                    .like("code", "__0000")
//                    .orderByAsc("sort")
//            ));
//        }
//        else {
//            if (dispCode.length()!=6) {
//                return R.failed("区域代码必须是6位");
//            }
//
//            if ("0000".equals(dispCode.substring(2, 6))) {
//                // 提供的是省代码, 取下属地州市
//                String cityCode = dispCode.substring(0, 2);
//                return R.ok(areaService.listAreaVO(Wrappers.<Area>query()
//                        .like("disp_code", cityCode+"__00")
//                        .ne("disp_code", cityCode+"0000")
//                ));
//            }
//            else if ("00".equals(dispCode.substring(4, 6))) {
//                // 提供的是地州市代码, 取下属区县
//                String cityCode = dispCode.substring(0, 4);
//                return R.ok(areaService.listAreaVO(Wrappers.<Area>query()
//                        .like("disp_code", cityCode+"__")
//                        .ne("disp_code", cityCode+"00")
//                ));
//            }
//            else {
//                List<AreaVO> empty = new ArrayList<AreaVO>();
//                return R.ok(empty);
//            }
//        }
//
//    }

    /**
     * 查询区域
     * 查询条件: id/parentId/name
     * @param area 地市
     * @return 区域VO 列表
     */
    @GetMapping("/vo")
//    @Cacheable(value = CacheConstants.AREA_SUB_DETAILS, key = "#code")
    public R<List<AreaVO>> listAreaVO(Area area) {
        return R.ok(areaService.listAreaVO(Wrappers.<Area>query()
                .like(StrUtil.isNotBlank(area.getName()), "name", area.getName())
                .eq(StrUtil.isNotBlank(area.getId()), "id", area.getId())
                .eq(StrUtil.isNotBlank(area.getParentId()), "parent_id", area.getParentId())
                .orderByAsc("sort")
        ));
    }

    /**
     * 根据地区名字查询地区id
     * @param provinceName
     * @param cityName
     * @param areaName
     * @return
     */
    @GetMapping("/name/{provinceName}/{cityName}/{areaName}")
    public R<Map<String, String>>  listAreaVOByName(@PathVariable("provinceName") String provinceName,
                                                    @PathVariable("cityName")String cityName,
                                                    @PathVariable("areaName")String areaName) {
        Map<String, String> areaMap = new HashMap<>();

        QueryWrapper<Area> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(StrUtil.isNotBlank(provinceName),"name", provinceName); // 先查询省的id
        Area province = areaService.getOne(queryWrapper);
        if (province != null) {
            // 添加查询到的省的id
            areaMap.put("provinceId", province.getId());

            // 判断是否是直辖市
            if (!provinceName.equals(cityName)) {
                // 查询市
                queryWrapper = new QueryWrapper<Area>();
                queryWrapper.eq("parent_id", province.getId());
                queryWrapper.eq(StrUtil.isNotBlank(cityName),"name", cityName); // 查询市

                Area city = areaService.getOne(queryWrapper);
                if (city != null) {
                    // 添加查询到的市的id
                    areaMap.put("cityId", city.getId());
                    if (areaName.equals(cityName)) {
                        areaMap.put("areaId", city.getId());
                    }
                } else {
                    throw new RuntimeException(String.format("找不到名字为 %s 的地区", cityName));
                }
            } else {
                // 该地区是直辖市
                // 添加查询到的省的id
                areaMap.put("cityId", province.getId());
            }


            if (areaMap.get("cityId") != null && !areaName.equals(cityName)) {
                // 查询区县的
                queryWrapper = new QueryWrapper<Area>();
                queryWrapper.eq("parent_id", areaMap.get("cityId"));
                queryWrapper.eq(StrUtil.isNotBlank(areaName),"name", areaName); // 查询区县的
                Area area = areaService.getOne(queryWrapper);
                if (area != null) {
                    // 添加查询到的区县的id
                    areaMap.put("areaId", area.getId());
                }
//                else {
//                    throw new RuntimeException(String.format("找不到名字为 %s 的地区", areaName));
//                }
            }
        } else {
            throw new RuntimeException(String.format("找不到名字为 %s 的地区", provinceName));
        }

        return R.ok(areaMap);
    }
}
