package digital.laboratory.platform.sys.controller;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.sys.entity.Deliverer;
import digital.laboratory.platform.sys.service.DelivererService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.security.Principal;
import java.util.List;


/**
 * 送检员
 *
 * @author Zhang Xiaolong created at  2022-08-16
 * @describe 送检员 前端控制器
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/comm/deliverer" )
@Api(value = "deliverer", tags = "送检员管理")
public class CommDelivererController {

	private final DelivererService delivererService;

	/**
	 * 通过名字查询送检员
	 *
	 * @param name
	 *
	 * @return R
	 */
	@ApiOperation(value = "通过名字查询送检员", notes = "通过名字查询送检员, 如果没有找到, 则返回数据为 null")
	@GetMapping("/name/{name}")
	public R<Deliverer> getByName(@PathVariable("name")String name, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		Deliverer deliverer = delivererService.getByName(name, dlpUser.getId());
		if (deliverer != null) {
			return R.ok(deliverer, "取送检员信息成功");
		}
		else {
			return R.failed(String.format("没有找到名字为 %s 的送检员", name));
		}
	}


	/**
	 * 通过id查询送检员
	 *
	 * @param id id
	 *
	 * @return R
	 */
	@ApiOperation(value = "通过id查询", notes = "通过id查询")
	@GetMapping("/{id}")
	public R<Deliverer> getById(@PathVariable("id") String id, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		Deliverer deliverer = delivererService.getById(id);
		if (deliverer != null) {
			if (StrUtil.equalsIgnoreCase(dlpUser.getId(), deliverer.getOwnerUserId())) {
				return R.ok(deliverer);
			}
			else {
				throw new RuntimeException(String.format("你没有权限访问id为 %s 的送检员的数据", id));
			}
		}
		else {
			return R.failed(String.format("没有找到id为 %s 的送检员", id));
		}
	}

	/**
	 * 列表查询
	 *
	 * @param name 查询条件
	 *
	 * @return
	 */
	@ApiOperation(value = "列表查询", notes = "列表查询\n" +
			"参数:\n" +
			"<pre>\n" +
			"name 送检员名字, 可以模糊查询, 支持 like %name% " +
			"</pre>\n" +
			"")
	@GetMapping("/list")
	public R<List<Deliverer>> getDelivererList(@RequestParam(value = "name", required = false) String name, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
		//deliverer.setOwnerUserId(dlpUser.getId());

		List<Deliverer> list = delivererService.list(Wrappers.<Deliverer>query()
				.eq("owner_user_id", dlpUser.getId())       // 只查询当前用户拥有的送检员
				.like(StrUtil.isNotBlank(name), "name", name)
				.orderByDesc("name")
		);
		return R.ok(list);
	}


	/**
	 * 分页查询
	 *
	 * @param page      分页对象
	 * @param deliverer 送检员
	 *
	 * @return
	 */
	@ApiOperation(value = "分页查询", notes = "分页查询")
	@GetMapping("/page")
	public R<IPage<Deliverer>> getDelivererPage(Page page, Deliverer deliverer, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
		deliverer.setOwnerUserId(dlpUser.getId());

		return R.ok(delivererService.page(page, Wrappers.<Deliverer>query()
				.eq("owner_user_id", dlpUser.getId())       // 只查询当前用户拥有的送检员
				.like(StrUtil.isNotBlank(deliverer.getName()), "name", deliverer.getName())
				.orderByDesc("name")
		));
	}


	// /**
	//  * 新增送检员
	//  *
	//  * @param deliverer 送检员
	//  *
	//  * @return R
	//  */
	// @ApiOperation(value = "新增送检员", notes = "新增送检员")
	// @SysLog("新增送检员")
	// @PostMapping
	// public R<Deliverer> postAddObject(@RequestBody Deliverer deliverer, HttpServletRequest theHttpServletRequest) {
	// 	Principal principal = theHttpServletRequest.getUserPrincipal();
	// 	DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
	// 	deliverer.setOwnerUserId(dlpUser.getId());
	// 	deliverer.setId(IdWorker.get32UUID().toUpperCase());
	//
	// 	if (delivererService.save(deliverer)) {
	// 		return R.ok(deliverer, "新增送检员成功");
	// 	}
	// 	else {
	// 		throw new RuntimeException("新增送检员失败");
	// 	}
	//
	// }

	/**
	 * 修改或创建送检员 更新存储的送检员信息
	 *
	 * @param deliverer 送检员
	 *
	 * @return R
	 */
	@ApiOperation(value = "修改送检员", notes = "修改送检员。如果送检员存在则更新, 如果不存在则新增")
	@SysLog("修改送检员")
	@PutMapping
	public R<Boolean> putUpdateById(@RequestBody Deliverer deliverer, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();


		deliverer.setOwnerUserId(dlpUser.getId());

		if (delivererService.renew(deliverer)) {
			return R.ok(true, "更新送检员信息成功");
		}
		else {
			//throw new RuntimeException(String.format("更新送检员[%s]信息出错", deliverer.getName()));
			return R.ok(false, String.format("更新送检员[%s]信息出错", deliverer.getName()));
		}
	}

    /**
     * 通过id删除送检员
     * @param id id
     * @return R
     */
    @ApiOperation(value = "通过id删除送检员", notes = "通过id删除送检员")
    @SysLog("通过id删除送检员" )
    @DeleteMapping("/{id}" )
    public R<Deliverer> deleteById(@PathVariable String id, HttpServletRequest theHttpServletRequest) {
	    Principal principal = theHttpServletRequest.getUserPrincipal();
	    DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

	    Deliverer oldDeliverer = delivererService.getById(id);
	    if (oldDeliverer != null) {
		    if (StrUtil.equalsIgnoreCase(dlpUser.getId(), oldDeliverer.getOwnerUserId())) {
			    if (delivererService.removeById(id)) {
				    return R.ok(oldDeliverer, "送检员删除成功");
			    }
			    else {
					throw new RuntimeException("送检员删除失败");
			    }
		    }
		    else {
				throw new RuntimeException(String.format("你没有权限删除id为 %s 的送检员", id));
		    }
	    }
	    else {
			throw new RuntimeException(String.format("没有找到id为 %s 的送检员", id));
	    }
    }

}
