package digital.laboratory.platform.sys.controller;

import cn.hutool.core.util.StrUtil;
import digital.laboratory.platform.sys.entity.Dictionary;
import digital.laboratory.platform.sys.vo.DictVueStoreVo;
import digital.laboratory.platform.sys.vo.DictionaryVO;
import digital.laboratory.platform.sys.service.DictionaryService;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.security.annotation.Inner;
import digital.laboratory.platform.common.security.util.SecurityUtils;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 * 字典表 前端控制器
 * </p>
 *
 * @author lengleng
 * @since 2019-03-19
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/comm/dictionary")
@Api(tags = "02-字典管理")
public class CommDictionaryController {
	private final DictionaryService dictionaryService;

	/**
	 * 取全部字典信息
	 * @return 字典列表
	 */
	@ApiOperation(value = "取字典列表", notes = "取字典列表")
	@GetMapping
	public R getDictionaryList() {
		return R.ok(dictionaryService.list(Wrappers.emptyWrapper()));
	}


	/**
	 * 返回树形字典集合
	 * @param parentId 父节点ID
	 * @return 树形权限
	 */
	@ApiOperation(value = "取字典分支树", notes = "取字典分支树")
	@ApiImplicitParams({
			@ApiImplicitParam(name="parentId",value="分支树的标识",required=true,paramType="query"),
	})
	@GetMapping(value = "/tree")
	public R getTree(String parentId) {
		return R.ok(dictionaryService.treeDictionary(parentId));
	}

	/**
	 * 内部调用
	 * @param parentId
	 * @return
	 */
	@GetMapping(value = "/insideTree")
	public List<Dictionary> insideTreeData(String parentId) {
		return dictionaryService.list(Wrappers.<Dictionary>lambdaQuery()
				.orderByAsc(Dictionary::getSort));
	}

	@GetMapping(value = "/treeParentData")
	public R getTreeParentData(String parentId) {
		return R.ok(dictionaryService.getTreeParentDictionary());
	}
	/**
	 * 通过ID查询字典信息
	 * @param id ID
	 * @return 字典信息
	 */
	@ApiOperation(value = "通过ID查询字典信息", notes = "通过ID查询字典信息")
	@ApiImplicitParams({
			@ApiImplicitParam(name="id",value="id",required=true,paramType="path"),
	})
	@GetMapping("/{id}")
	public R getById(@PathVariable String id) {
		//throw new RuntimeException("搞个异常试一下");
		return R.ok(dictionaryService.getById(id));
	}
	@GetMapping("/getDictByID")
	public Dictionary getDictByID(String dictID)
	{
		return dictionaryService.getById(dictID);
	}
	/**
	 * 通过ID查询字典信息
	 * @param id ID
	 * @return 字典信息
	 */
	@ApiOperation(value = "通过ID查询字典信息, 仅供内部模块调用", notes = "通过ID查询字典信息, 仅供内部模块调用")
	@ApiImplicitParams({
			@ApiImplicitParam(name="id",value="id",required=true,paramType="path"),
	})
	@GetMapping("/inner/{id}")
	@Inner
	public R innerGetById(@PathVariable String id) {
		return R.ok(dictionaryService.getById(id));
	}

	/**
	 * 通过字典类型查找字典
	 * @param type 类型
	 * @return 同类型字典
	 */
	@GetMapping("/inner/type/{type}")
	@Inner
//	@Cacheable(value = CacheConstants.DICT_DETAILS, key = "#type")
	public R<List<Dictionary>> innerGetDictionaryByType(@PathVariable String type) {
		return R.ok(dictionaryService.list(Wrappers.<Dictionary>query()
				.eq("type", type)
				.orderByAsc("sort")
				.orderByAsc("id")));
//		return R.ok(sysDictionaryService.list(Wrappers.<Dictionary>query().lambda().like(true, "type", type, SqlLike.DEFAULT)));
	}

	/**
	 * 分页查询字典信息
	 * @param page 分页对象
	 * @return 分页对象
	 */
	@ApiOperation(value = "分页查询字典信息", notes = "分页查询字典信息\n"+
			"查询条件:\n"+
			"page	分页信息\n"+
			"dict	查询条件组成的字典对象, 具体有以下几个属性:\n"+
			"	module/type/code/label/description/comments 这些查询条件都是 like 方式\n" +
			"	这几个查询条件之间是 and 关系。\n" +
			"例如 type=abc 表示查询字典type属性中包含abc的全部字典\n" +
			"例如 type=abc&code=def 表示查询字典type属性中包含abc并且code属性中包含def的全部字典\n"+
			"实际调用时, 并不是直接提供page和dict对象, 而是提供page的属性和dict的属性, 后端接口程序会自动组装成对象\n"+
			""
	)
	@ApiImplicitParams({
			@ApiImplicitParam(name="page",value="page",required=true,paramType="query"),
			@ApiImplicitParam(name="dict",value="查询条件",required=false,paramType="query"),
	})
	@GetMapping("/page")
	public R<IPage> getDictionaryPage(Page page, Dictionary dict) {
		page.setOrders(OrderItem.ascs("type", "code"));
		//System.out.println(String.format("CommDictionaryController.getDictPage(), dict=%s", dict.toString()));
		//System.out.println(String.format("CommDictionaryController.getDictPage(), page=%s", page.toString()));


		return R.ok(dictionaryService.page(page, Wrappers.<Dictionary>query()
				.like(dict.getModule()!=null,"module", dict.getModule())
				.like(dict.getType()!=null,"type", dict.getType())
				.like(dict.getCode()!=null,"code", dict.getCode())
				.like(dict.getLabel()!=null,"label", dict.getLabel())
				.like(dict.getDescription()!=null,"description", dict.getDescription())
				.like(dict.getComments()!=null,"comments", dict.getComments())
		));


		//return R.ok(sysDictionaryService.page(page, Wrappers.query(dict)));
	}


	/**
	 * 取全部类型
	 * @return 同类型字典
	 */
	@GetMapping("/types")
//	@Cacheable(value = CacheConstants.DICT_DETAILS, key = "#type")
	public R<List<Dictionary>> getDictionaryTypes(String module) {
		List<Dictionary> list = dictionaryService.listTypes(module);
		return R.ok(list);
	}

	/**
	 * 通过字典类型查找字典
	 * @param type 类型
	 * @return 同类型字典
	 */
	@GetMapping("/type/{type}")
//	@Cacheable(value = CacheConstants.DICT_DETAILS, key = "#type")
	public R<List<Dictionary>> getDictionaryByType(@PathVariable String type) {
		return R.ok(dictionaryService.list(Wrappers.<Dictionary>query()
				.eq("type", type)
				//.notLike("label","常用")
				.orderByAsc("sort")
				.orderByAsc("id")));
	}
	//一次查出多个字典的值,比如页面刚加载的时候，就希望一次加载多个字典到Vue store 中
	@GetMapping("/getVueStoreDictList")
	public R<List<DictVueStoreVo>> getDictByInit(@RequestParam List<String> typeList)
	{
		long beginTime1=System.currentTimeMillis();
		List<DictVueStoreVo> retData=dictionaryService.getVueStoreLoadDictList(typeList);
		long beginTime2=System.currentTimeMillis();
		System.out.println(beginTime1+"#"+beginTime2);
		return R.ok(retData);
	}
	/**
	 * 通过字典类型查找字典
	 * @param type 类型
	 * @return 同类型字典
	 */
	@GetMapping("/typeLike/{type}")
//	@Cacheable(value = CacheConstants.DICT_DETAILS, key = "#type")
	public R getDictionaryByTypeLike(@PathVariable String type) {
		return R.ok(dictionaryService.list(Wrappers.<Dictionary>query()
				.like("type", type)
				.orderByAsc("sort")
				.orderByAsc("id")));
	}

	/**
	 * 添加字典
	 * @param dictionary 字典信息
	 * @return success、false
	 */
	@SysLog("添加字典")
	@PostMapping
	@PreAuthorize("@pms.hasPermission('sys_dict_add')")
	public R add(@Valid @RequestBody Dictionary dictionary) {
//		String username = SecurityUtils.getUser().getUsername();
//		dictionary.setCreateBy(username);
//		dictionary.setUpdateBy(username);
		boolean r = dictionaryService.save(dictionary);
		if (r) {
			return R.ok("添加字典项成功");
		}
		else {
			return R.failed("添加字典项失败");
		}
	}

	/**
	 * 删除字典, 并且清除字典缓存
	 * @param id ID
	 * @return R
	 */
	@SysLog("删除字典")
	@DeleteMapping("/{id}")
	@PreAuthorize("@pms.hasPermission('sys_dict_del')")
	public R removeById(@PathVariable String id) {
		//dictionaryService.removeDictionary(id);
		dictionaryService.deleteDictionary(id);
		return R.ok("删除字典项成功");
	}

	/**
	 * 修改字典
	 * @param dictionary 字典信息
	 * @return success/false
	 */
	@PutMapping
	@SysLog("修改字典")
	//@PreAuthorize("@pms.hasPermission('sys_dict_edit')")
	public R updateById(@Valid @RequestBody Dictionary dictionary) {
		String username = SecurityUtils.getUser().getUsername();
		dictionary.setUpdateBy(username);
		dictionaryService.updateDictionary(dictionary);
		return R.ok("修改字典项成功");
	}

	/**
	 * 字典列表
	 * 查询条件: id/parentId/name
	 * @param dictionary 字典id
	 * @return 字典VO 列表
	 */
	@GetMapping("/vo")
	@Inner
	public R<List<DictionaryVO>> listDictionaryVO(Dictionary dictionary) {
		return R.ok(dictionaryService.listDictionaryVO(Wrappers.<Dictionary>query()
				.like(StrUtil.isNotBlank(dictionary.getLabel()), "label", dictionary.getLabel())
				.eq(StrUtil.isNotBlank(dictionary.getId()), "id", dictionary.getId())
				.eq(StrUtil.isNotBlank(dictionary.getType()), "type", dictionary.getType())
				.orderByAsc("sort")
		));
	}
	/**
	 * 内部调用添加字典
	 * @return success、false
	 */
	@SysLog("内部调用查询字典")
	@GetMapping("/inner/query")
	public R queryDictionaryVO(@RequestParam(value = "label") String label,
									@RequestParam(value = "id") String id) {
		// 根据label 和 type 去查询字典
		List<DictionaryVO> dictionaryVOS = dictionaryService.listDictionaryVO(Wrappers.<Dictionary>query()
				.eq(StrUtil.isNotBlank(label), "label", label)
				.like(StrUtil.isNotBlank(id), "type", id)
				.orderByAsc("sort")
		);

		// 查询不到则把这个新的类型添加到字典里
		if (dictionaryVOS == null || dictionaryVOS.size() == 0 || dictionaryVOS.get(0) == null) {

			Dictionary dictionary = dictionaryService.addThirdPartyDictionary(label, id);
			if (dictionary != null) {
				return R.ok(dictionary.getId());
			} else {
				return R.failed("添加字典失败");
			}
		}

		return R.ok(dictionaryVOS.get(0).getId());
	}
	@GetMapping("/oldDictTreeVo")
	public R<List<DictionaryVO>> oldDictTreeVo(String parentId) {
		return R.ok(dictionaryService.listDictionaryVO(Wrappers.<Dictionary>query()
				.eq(StrUtil.isNotBlank(parentId), "type", parentId)
				//.notLike("label","常用")
				.orderByAsc("sort")
		));
	}
	@PutMapping("/replayClient")
	public  R replayClient(){
		return R.ok();
	}
}
