package digital.laboratory.platform.sys.controller;

import cn.hutool.core.io.FileUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.StrUtil;
import cn.hutool.poi.excel.ExcelReader;
import cn.hutool.poi.excel.ExcelUtil;
import com.alibaba.cloud.commons.lang.StringUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.security.annotation.Inner;
import digital.laboratory.platform.sys.entity.Drug;
import digital.laboratory.platform.sys.entity.DrugLite;
import digital.laboratory.platform.sys.entity.DrugType;
import digital.laboratory.platform.sys.service.DrugService;
import digital.laboratory.platform.sys.service.DrugTypeService;
import digital.laboratory.platform.sys.vo.SuspiciousDrugVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.io.File;
import java.text.ParseException;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;


/**
 * 毒品清单
 *
 * @author Zhang Xiaolong created at  2022-06-23
 * @describe 毒品清单 前端控制器
 */
@RestController
@RequestMapping("/comm/drug" )
@Api(value = "drug", tags = "毒品清单管理")
public class CommDrugController {

    @Resource
    private DrugService drugService;

    @Resource
    private DrugTypeService drugTypeService;
	/**
	 * 通过id查询毒品清单
	 * @param id id
	 * @return R
	 */
	@ApiOperation(value = "通过id查询", notes = "通过id查询")
	@GetMapping("/{id}" )
//	@PreAuthorize("@pms.hasPermission('common_drug_get')" )
	public R<Drug> getById(@PathVariable("id" ) String id) {
		return R.ok(drugService.getById(id));
	}

    /**
     * 提供给内部服务通过id查询
     * @param id id
     * @return R
     */
    @ApiOperation(value = "提供给内部服务通过id查询", notes = "提供给内部服务通过id查询")
    @GetMapping("/inner/{id}" )
    @Inner(value = false)
    public R<Drug> innerGetById(@PathVariable("id" ) String id) {
        return R.ok(drugService.getById(id));
    }

    @ApiOperation("获取所有顶层毒品类型的数据，以map的列表返回")
    @GetMapping("/drugMapList")
    public R< List<SuspiciousDrugVO>> drugMapList(String drugType, String name, String englishName, String casCode) {
        return R.ok(drugService.drugMapList(drugType, name, englishName, casCode));
    }


    /**
     * 通过毒品名称查询毒品清单
     * @param name name
     * @return R
     */
    @ApiOperation(value = "通过名称查询", notes = "通过名称查询")
    @GetMapping("/name/{name}" )
//	@PreAuthorize("@pms.hasPermission('common_drug_get')" )
    public R<DrugLite> getByName(@PathVariable("name" ) String name) {
        if (StrUtil.isBlank(name)) {
            throw new RuntimeException("名称不能为空");
        }

        List<Drug> list = drugService.list(Wrappers.<Drug>query().eq("name", name));
        if (list == null || list.size() == 0) {
            return R.ok();
        }
        Drug drug = list.get(0);
        DrugLite drugLite = new DrugLite();
        final BeanCopier beanCopier = BeanCopier.create(Drug.class, DrugLite.class, false);
        beanCopier.copy(drug, drugLite, null);
        return R.ok(drugLite);
    }

	/**
	 * 查询毒品列表
	 * @param drug 查询条件(name, alias => like)
	 * @return
	 */
	@ApiOperation(value = "常用毒品查询列表", notes = "常用毒品查询列表")
	@GetMapping("/list" )
//    @PreAuthorize("@pms.hasPermission('common_drug_get')" )
	public R getDrugPage(Drug drug) {
        List<DrugType>  drugTypeList=drugTypeService.list(Wrappers.<DrugType>lambdaQuery()
                .eq(DrugType::getName,"常规毒品"));
        Assert.notEmpty(drugTypeList,"系统中并没有找到 常规毒品 分类");

        List<Drug> r = drugService.list(Wrappers.<Drug>query()
				.like(StrUtil.isNotBlank(drug.getName()), "name", drug.getName())
				.like(StrUtil.isNotBlank(drug.getAlias()), "english_name", drug.getEnglishName())
                .like(StrUtil.isNotBlank(drug.getCasCode()), "cas_code", drug.getCasCode())
                .eq("drug_type", drugTypeList.get(0).getId())//只查询NPS的毒品列表
		);
		return R.ok(r);
	}
    //NPS 毒品清单
    @ApiOperation(value = "查询NPS的毒品列表",notes = "查询NPS的毒品列表")
    @GetMapping("/getNPSDrugList")
    public R getNPSDrugList(Drug drug){
        List<DrugType>  drugTypeList=drugTypeService.list(Wrappers.<DrugType>lambdaQuery()
                .eq(DrugType::getName,"常规毒品"));
        Assert.notEmpty(drugTypeList,"系统中并没有找到 NPS毒品 分类");
        //根据这个ID查询出下面的子类ID
        List<Drug> r = drugService.list(Wrappers.<Drug>query()
                .like(StrUtil.isNotBlank(drug.getName()), "name", drug.getName())
                .like(StrUtil.isNotBlank(drug.getEnglishName()), "english_name", drug.getEnglishName())
                .like(StrUtil.isNotBlank(drug.getCasCode()), "cas_code", drug.getCasCode())
                .like(StrUtil.isNotBlank(drug.getCode()), "code", drug.getCode())
                .eq(drug.getPublishTime()!=null, "publish_time", drug.getPublishTime())
                .eq(drug.getImplementTime()!=null, "implement_time", drug.getImplementTime())
                .ne("drug_type", drugTypeList.get(0).getId())//只查询NPS的毒品列表

        );
        return R.ok(r);
    }
	/**
     * 分页查询
     * @param page 分页对象
     * @param drug 毒品清单
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
//    @PreAuthorize("@pms.hasPermission('common_drug_get')" )
    public R<IPage<Drug>> getDrugPage(Page page, Drug drug) {
        return R.ok(drugService.getDrugPage(page,drug));
    }


    /**
     * 新增毒品清单
     * @param drug 毒品清单
     * @return R
     */
    @ApiOperation(value = "新增毒品清单", notes = "新增毒品清单")
    @SysLog("新增毒品清单" )
    @PostMapping
//    @PreAuthorize("@pms.hasPermission('common_drug_add')" )
    public R postAddObject(@RequestBody Drug drug) {
        return R.ok(drugService.addDrug(drug));
    }

    /**
     * 修改毒品清单
     * @param drug 毒品清单
     * @return R
     */
    @ApiOperation(value = "修改毒品清单", notes = "修改毒品清单")
    @SysLog("修改毒品清单" )
    @PutMapping
//    @PreAuthorize("@pms.hasPermission('common_drug_edit')" )
    public R putUpdateById(@RequestBody Drug drug) {
        return R.ok(drugService.updateById(drug));
    }
    /**
     * 通过id删除毒品清单
     * @param id id
     * @return R
     */
    @ApiOperation(value = "通过id删除毒品清单", notes = "通过id删除毒品清单")
    @SysLog("通过id删除毒品清单" )
    @DeleteMapping("/{id}" )
//    @PreAuthorize("@pms.hasPermission('common_drug_del')" )
    public R deleteById(@PathVariable String id) {
        return R.ok(drugService.removeById(id));
    }
    //导入NPS毒品筛查列表
    @PostMapping("/importNPSExcelData")
    public R importNPSExcelData() throws ParseException {
        File file= FileUtil.file("D:\\禁毒大数据平台\\整理之后的数据.xlsx");
        ExcelReader excelReader=ExcelUtil.getReader(file);
        List<List<Object>> excelDataList=excelReader.read();
        List<Drug> drugList=new ArrayList<>();
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        for(int i=0;i<excelDataList.size();i++){
            if(i>1){
                //0=编号  1=名称 2=英文名称 3=cas编号  4=发布日期
                //5=实施日期 6=来源，7=备注
                Drug drug=new Drug();
                drug.setId(IdWorker.get32UUID());
                drug.setCode(excelDataList.get(i).get(0).toString().trim());
                drug.setName(excelDataList.get(i).get(1).toString().trim());
                drug.setEnglishName(excelDataList.get(i).get(2).toString().trim());
                drug.setCasCode(excelDataList.get(i).get(3)!=null?excelDataList.get(i).get(3).toString().trim():"");
                drug.setPublishTime(LocalDateTime.parse(excelDataList.get(i).get(4).toString().trim()+" 00:00:00",formatter));
                drug.setImplementTime(LocalDateTime.parse(excelDataList.get(i).get(5).toString().trim()+" 00:00:00",formatter));
                drug.setSource(excelDataList.get(i).get(6).toString().trim());
                drug.setComments(excelDataList.get(i).get(7)!=null?excelDataList.get(i).get(7).toString().trim():"");
                drug.setOrderIndex(Integer.parseInt(excelDataList.get(i).get(0).toString().trim()));
                drug.setDrugType("1");
                System.out.println("添加完了一条，编号是：--------------------------------------"+drug.getCode());
                drugList.add(drug);
            }
        }
        return R.ok(drugService.saveBatch(drugList));
    }

}
