package digital.laboratory.platform.sys.controller;

import cn.hutool.core.lang.Assert;
import com.alibaba.cloud.commons.lang.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.sys.entity.Drug;
import digital.laboratory.platform.sys.entity.DrugType;
import digital.laboratory.platform.sys.service.DrugService;
import digital.laboratory.platform.sys.service.DrugTypeService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;


/**
 * 毒品分类
 *
 * @author Zhang Xiaolong created at  2022-06-23
 * @describe 毒品分类 前端控制器
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/comm/drug_type" )
@Api(value = "drug_type", tags = "毒品分类管理")
public class CommDrugTypeController {

    private final DrugTypeService drugTypeService;
    @Resource
    private DrugService drugService;

	/**
	 * 通过id查询毒品分类
	 * @param id id
	 * @return R
	 */
	@ApiOperation(value = "通过id查询", notes = "通过id查询")
	@GetMapping("/{id}" )
//	@PreAuthorize("@pms.hasPermission('common_drug_type_get')" )
	public R getById(@PathVariable("id" ) String id) {
		return R.ok(drugTypeService.getById(id));
	}

	/**
     * 分页查询
     * @param page 分页对象
     * @param drugType 毒品分类
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
//    @PreAuthorize("@pms.hasPermission('common_drug_type_get')" )
    public R getDrugTypePage(Page page, DrugType drugType) {

        return R.ok(drugTypeService.page(page, Wrappers.query(drugType)));
    }


    /**
     * 新增毒品分类
     * @param drugType 毒品分类
     * @return R
     */
    @ApiOperation(value = "新增毒品分类", notes = "新增毒品分类")
    @SysLog("新增毒品分类" )
    @PostMapping
//    @PreAuthorize("@pms.hasPermission('common_drug_type_add')" )
    public R postAddObject(@RequestBody DrugType drugType) {
        Assert.notBlank(drugType.getName(),"类别名称不能为空");
        Assert.notBlank(drugType.getPid(),"父节点ID不能为空");
        return R.ok(drugTypeService.addDrugType(drugType));
    }

    /**
     * 修改毒品分类
     * @param drugType 毒品分类
     * @return R
     */
    @ApiOperation(value = "修改毒品分类", notes = "修改毒品分类")
    @SysLog("修改毒品分类" )
    @PutMapping
//    @PreAuthorize("@pms.hasPermission('common_drug_type_edit')" )
    public R putUpdateById(@RequestBody DrugType drugType) {
        Assert.notBlank(drugType.getName(),"类别名称不能为空");
        List<DrugType> list=drugTypeService.list(Wrappers.<DrugType>lambdaQuery()
                .eq(DrugType::getName,drugType.getName())
                .ne(DrugType::getId,drugType.getId()));
        if(list.size()>0){
            return R.failed("类别名称不能重复");
        }
        return R.ok(drugTypeService.updateById(drugType));
    }

    /**
     * 通过id删除毒品分类
     * @param id id
     * @return R
     */
    @ApiOperation(value = "通过id删除毒品分类", notes = "通过id删除毒品分类")
    @SysLog("通过id删除毒品分类" )
    @DeleteMapping("/{id}" )
//    @PreAuthorize("@pms.hasPermission('common_drug_type_del')" )
    public R deleteById(@PathVariable String id) {
        //删除的时候，需要判断这个类别下是否有毒品
        List<Drug>  drugList= drugService.list(Wrappers.<Drug>lambdaQuery()
                .eq(Drug::getDrugType,id));
        if(drugList.size()>0){
            return R.failed("该分类下还有毒品清单，不能删除，请先移除该分类下的毒品清单");
        }
        return R.ok(drugTypeService.removeById(id));
    }

    // new add 8-30
    //以树的形式将类型组织数据，方便UI前端显示
    @ApiOperation(value = "通过Pid获取结构树", notes = "通过Pid获取结构树")
    @GetMapping("/getDrugTypeTreeData")
    public R getDrugTypeTreeData(String pid){
        if(StringUtils.isBlank(pid)){
            pid="0";
        }
        return R.ok(drugTypeService.getDrugTypeTreeData(pid),"获取数据成功");
    }

}
