/*
 *    Copyright (c) 2018-2025, lengleng All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * Neither the name of the pig4cloud.com developer nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * Author: lengleng (wangiegie@gmail.com)
 */

package digital.laboratory.platform.sys.controller;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.security.annotation.Inner;
import digital.laboratory.platform.sys.entity.PublicParam;
import digital.laboratory.platform.sys.service.PublicParamService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.security.SecurityRequirement;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpHeaders;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

/**
 * 公共参数
 *
 * @author Lucky
 * @date 2019-04-29
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/comm/public_param")
@Tag(name = "公共参数配置")
@SecurityRequirement(name = HttpHeaders.AUTHORIZATION)
public class CommPublicParamController {

	private final PublicParamService publicParamService;

	/**
	 * 通过key查询公共参数值
	 * @param publicKey
	 * @return
	 */
	@Inner(value = false)
	@Operation(summary = "查询公共参数值", description = "根据key查询公共参数值")
	@GetMapping("/public_value/{publicKey}")
	public R<String> innerPublicKey(@PathVariable("publicKey") String publicKey) {
		return R.ok(publicParamService.getSysPublicParamKeyToValue(publicKey));
	}

	/**
	 * 分页查询
	 * @param page 分页对象
	 * @param publicParam 公共参数
	 * @return
	 */
	@Operation(summary = "分页查询", description = "分页查询, 查询参数:\n" +
			"page - 分页参数\n" +
			"paramName - 参数名称, like 模糊查询\n" +
			"paramKey - 参数键, like 模糊查询")
	@GetMapping("/page")
	public R getSysPublicParamPage(Page page, PublicParam publicParam) {
		// return R.ok(publicParamService.page(page,
		// 		Wrappers.<PublicParam>lambdaQuery()
		// 				.like(StrUtil.isNotBlank(publicParam.getParamName()), PublicParam::getParamName,
		// 						publicParam.getParamName())
		// 				.like(StrUtil.isNotBlank(publicParam.getParamKey()), PublicParam::getParamKey,
		// 						publicParam.getParamKey())));

		return R.ok(publicParamService.page(page, Wrappers.<PublicParam>query()
						.like(StrUtil.isNotBlank(publicParam.getParamName()), "param_name", publicParam.getParamName())
						.like(StrUtil.isNotBlank(publicParam.getParamKey()), "param_key", publicParam.getParamKey())));
	}

	/**
	 * 通过id查询公共参数
	 * @param publicId id
	 * @return R
	 */
	@Operation(summary = "通过id查询公共参数", description = "通过id查询公共参数")
	@GetMapping("/{publicId}")
	public R<PublicParam> getById(@PathVariable("publicId") Long publicId) {
		return R.ok(publicParamService.getById(publicId));
	}

	/**
	 * 新增公共参数
	 * @param publicParam 公共参数
	 * @return R
	 */
	@Operation(summary = "新增公共参数", description = "新增公共参数")
	@SysLog("新增公共参数")
	@PostMapping
	@PreAuthorize("@pms.hasPermission('sys_publicparam_add')")
	public R post_AddNewParam(@RequestBody PublicParam publicParam) {
		return R.ok(publicParamService.save(publicParam));
	}

	/**
	 * 修改公共参数
	 * @param publicParam 公共参数
	 * @return R
	 */
	@Operation(summary = "修改公共参数", description = "修改公共参数")
	@SysLog("修改公共参数")
	@PutMapping
	@PreAuthorize("@pms.hasPermission('sys_publicparam_edit')")
	public R put_UpdateById(@RequestBody PublicParam publicParam) {
		return publicParamService.updateParam(publicParam);
	}

	/**
	 * 通过id删除公共参数
	 * @param publicId id
	 * @return R
	 */
	@Operation(summary = "删除公共参数", description = "删除公共参数")
	@SysLog("删除公共参数")
	@DeleteMapping("/{publicId}")
	@PreAuthorize("@pms.hasPermission('sys_publicparam_del')")
	public R delete_RemoveById(@PathVariable Long publicId) {
		return publicParamService.removeParam(publicId);
	}

	/**
	 * 同步参数
	 * @return R
	 */
	@SysLog("同步参数")
	@PutMapping("/sync")
	public R sync() {
		return publicParamService.syncParamCache();
	}

}
