package digital.laboratory.platform.sys.controller;

import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.sys.entity.SysMenu;
import digital.laboratory.platform.sys.service.SysMenuService;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import io.swagger.annotations.Api;
import lombok.RequiredArgsConstructor;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.security.Principal;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 菜单管理控制器
 * @date 2017/10/31
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/sys/menu")
@Api(tags = "15-菜单管理", description="")
public class MenuController {

	private final SysMenuService sysMenuService;

	/**
	 * 返回当前用户的树形菜单集合
	 * @param parentId 父节点ID
	 * @return 当前用户的树形菜单
	 */
	@GetMapping
	public R getUserMenu(String parentId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser)((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

//		System.out.println(String.format("GET getUserMenu() /: parentId=%s",parentId));
		// 获取符合条件的菜单
//		Set<SysMenu> sysMenuList = sysMenuService
//				.findMenuByRoleIds(SecurityUtils.getRoles());

		Set<SysMenu> sysMenuList = sysMenuService
				.findMenuByUserId(dlpUser.getId());

		return R.ok(sysMenuService.filterMenu(sysMenuList, parentId));
	}

	/**
	 * 返回树形菜单集合
	 * @param menu 父节点的 menu
	 * @return 树形菜单
	 */
	@GetMapping(value = "/treemenu")
	public R getTreeMenu(String menu) {
		System.out.println(String.format("GET getTreeMenu() /treemenu: menu=%s",menu));
		return R.ok(sysMenuService.treeMenuByMenu(menu));
	}

	/**
	 * 返回树形菜单集合
	 * @param lazy 是否是懒加载
	 * @param parentId 父节点ID
	 * @return 树形菜单
	 */
	@GetMapping(value = "/tree")
	public R getTree(boolean lazy, String parentId) {
		System.out.println(String.format("GET getTree() /tree: parentId=%s",parentId));
		return R.ok(sysMenuService.treeMenuByParentId(lazy, parentId));
	}

	/**
	 * 返回岗位的菜单集合
	 * @param roleId 岗位ID
	 * @return 属性集合
	 */
	@GetMapping("/tree/{roleId}")
	public R getMenusByRoleId(@PathVariable String roleId) {
		System.out.println(String.format("GET getRoleTree() /tree/{roleId}: roleId=%s",roleId));
		List<String> roleIds = new ArrayList<String>();
		roleIds.add(roleId);
		return R.ok(sysMenuService.findMenuByRoleIds(roleIds).stream().map(SysMenu::getMenuId)
				.collect(Collectors.toList()));
	}

	/**
	 * 通过ID查询菜单的详细信息
	 * @param id 菜单ID
	 * @return 菜单详细信息
	 */
	@GetMapping("/{id}")
	public R getById(@PathVariable String id) {
		System.out.println(String.format("GET getById() /{id}: id=%s",id));
		return R.ok(sysMenuService.getById(id));
	}

	/**
	 * 新增菜单
	 * @param sysMenu 菜单信息
	 * @return 含ID 菜单信息
	 */
	@SysLog("新增菜单")
	@PostMapping
	@PreAuthorize("@pms.hasPermission('sys_menu_add')")
	public R addMenu(@Valid @RequestBody SysMenu sysMenu) {
		sysMenuService.save(sysMenu);
		return R.ok(sysMenu);
	}

	/**
	 * 删除菜单
	 * @param id 菜单ID
	 * @return success/false
	 */
	@SysLog("删除菜单")
	@DeleteMapping("/{id}")
	@PreAuthorize("@pms.hasPermission('sys_menu_del')")
	public R removeById(@PathVariable String id) {
		return R.ok(sysMenuService.removeMenuById(id));
	}

	/**
	 * 更新菜单
	 * @param sysMenu
	 * @return
	 */
	@SysLog("更新菜单")
	@PutMapping
	@PreAuthorize("@pms.hasPermission('sys_menu_edit')")
	public R update(@Valid @RequestBody SysMenu sysMenu) {
		return R.ok(sysMenuService.updateMenuById(sysMenu));
	}

}
