package digital.laboratory.platform.sys.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.sys.entity.Message;
import digital.laboratory.platform.sys.service.MessageService;
import digital.laboratory.platform.sys.vo.MessageVO;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.security.Principal;
import java.util.List;

/**
 * 消息
 *
 * @author Zhang Xiaolong created at  2023-02-02
 * @describe 消息 前端控制器
 *
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 *     为前端提供数据, 接受前端的数据
 *     为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 *     接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/comm/message" )
@Api(value = "message", tags = "消息管理")
public class MessageController {

    private final MessageService messageService;

	/**
	 * 通过id查询消息
	 * @param id id
	 * @return R
	 */
	@ApiOperation(value = "通过id获取消息", notes = "通过id获取消息")
	@GetMapping("/{id}" )
	public R<Message> getById(@PathVariable("id" ) String id, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        Message message = messageService.getMessageById(id, dlpUser.getId());
		return R.ok(message);
	}

	/**
     * 分页查询
     * @param page 分页对象
     * @param text 查询消息内容
	 * @param isread 是否已读
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
    public R<IPage<MessageVO>> getMessagePage(Page<Message> page, String text, Boolean isread, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		IPage<MessageVO> messageList = messageService.getMessagePage(page, text, isread, dlpUser.getId());
		return R.ok(messageList);
    }

	/**
	 * 列表查询
	 * @param text 查询消息内容
	 * @param isread 是否已读
	 * @return
	 */
	@ApiOperation(value = "列表查询", notes = "列表查询")
	@GetMapping("/list" )
	public R<List<MessageVO>> getMessageList(String text, Boolean isread, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		List<MessageVO> messageList = messageService.getMessageList(text, isread, dlpUser.getId());
		return R.ok(messageList);
	}

	/**
	 * 发送消息
	 * @param receiverId 收件人id
	 * @param msg 消息内容
	 * @return R
	 */
	@ApiOperation(value = "发送消息", notes = "发送消息")
	@SysLog("发送消息" )
	@PostMapping("/send_to_user/{receiverId}" )
	public R postSendMessageToUser(@RequestBody String msg, @PathVariable("receiverId") String receiverId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
		if (messageService.sendMessageToUser(receiverId, msg, dlpUser.getId())) {
			return R.ok("消息发送成功");
		}
		else {
			return R.ok("消息发送失败");
		}
	}

	/**
	 * 发送消息
	 * @param orgId
	 * @param includeSubOrg
	 * @param permissions
	 * @param msg 消息内容
	 * @return R
	 */
	@ApiOperation(value = "发送消息", notes = "发送消息")
	@SysLog("发送消息" )
	@PostMapping("/send_by_org_permission" )
	public R postSendMessage(@RequestBody String msg,
							 @RequestParam(value = "orgId", required = true) String orgId,
							 @RequestParam(value = "includeSubOrg", required = false) Boolean includeSubOrg,
							 @RequestParam(value = "permissions", required = false) List<String> permissions,
							 HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
		if (messageService.sendMessage(orgId, includeSubOrg, permissions, msg, dlpUser.getId())) {
			return R.ok("消息发送成功");
		}
		else {
			return R.ok("消息发送失败");
		}
	}


    // /**
    //  * 新增消息
    //  * @param message 消息
    //  * @return R
    //  */
    // @ApiOperation(value = "新增消息", notes = "新增消息")
    // @SysLog("新增消息" )
    // @PostMapping
    // @PreAuthorize("@pms.hasPermission('common_message_add')" )
    // public R<Message> postAddObject(@RequestBody Message message, HttpServletRequest theHttpServletRequest) {
	// 	Principal principal = theHttpServletRequest.getUserPrincipal();
	// 	DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
	//
    //     message.setId(IdWorker.get32UUID().toUpperCase());
	// 	if (messageService.save(message)) {
	// 		return R.ok(message, "对象创建成功");
	// 	}
	// 	else {
	// 		return R.failed(message, "对象创建失败");
	// 	}
    // }

    // /**
    //  * 修改消息
    //  * @param message 消息
    //  * @return R
    //  */
    // @ApiOperation(value = "修改消息", notes = "修改消息")
    // @SysLog("修改消息" )
    // @PutMapping
    // @PreAuthorize("@pms.hasPermission('common_message_edit')" )
    // public R<Message> putUpdateById(@RequestBody Message message, HttpServletRequest theHttpServletRequest) {
	// 	Principal principal = theHttpServletRequest.getUserPrincipal();
	// 	DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
	//
	// 	if (messageService.updateById(message)) {
	// 		return R.ok(message, "保存对象成功");
	// 	}
	// 	else {
	// 		return R.failed(message, "保存对象失败");
	// 	}
    // }

    /**
     * 通过id删除消息
     * @param id id
     * @return R
     */
    @ApiOperation(value = "通过id删除消息", notes = "通过id删除消息")
    @SysLog("通过id删除消息" )
    @DeleteMapping("/{id}" )
    @PreAuthorize("@pms.hasPermission('common_message_del')" )
    public R<Message> deleteById(@PathVariable String id, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        Message oldMessage = messageService.getById(id);

		if (messageService.removeById(id)) {
			return R.ok(oldMessage, "对象删除成功");
		}
		else {
			return R.failed(oldMessage, "对象删除失败");
		}

    }

}
