package digital.laboratory.platform.sys.controller;

import digital.laboratory.platform.sys.entity.SysPermission;
import digital.laboratory.platform.sys.service.SysPermissionService;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.security.util.SecurityUtils;
import io.swagger.annotations.Api;
import lombok.RequiredArgsConstructor;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

@RestController
@RequiredArgsConstructor
@RequestMapping("/sys/permission")
@Api(tags = "13-权限管理", description="权限管理")
public class PermissionController {

	private final SysPermissionService sysPermissionService;

	/**
	 * 返回当前用户的树形权限集合
	 * @param parentId 父节点ID
	 * @return 当前用户的树形权限
	 */
	@GetMapping
	public R getUserPermission(String parentId) {
		// 获取符合条件的权限
		Set<SysPermission> sysPermissionList = sysPermissionService
				.findPermissionByRoleId(SecurityUtils.getRoles());
		return R.ok(sysPermissionService.filterPermission(sysPermissionList, parentId));
	}

	/**
	 * 返回树形权限集合
	 * @param lazy 是否是懒加载
	 * @param parentId 父节点ID
	 * @return 树形权限
	 */
	@GetMapping(value = "/tree")
	public R getTree(boolean lazy, String parentId) {
		return R.ok(sysPermissionService.treePermission(lazy, parentId));
	}

	/**
	 * 返回岗位的权限集合
	 * @param roleId 岗位ID
	 * @return 属性集合
	 */
	@GetMapping("/tree/{roleId}")
	public R getRoleTree(@PathVariable String roleId) {
		List<String> roleIds = new ArrayList<String>();
		roleIds.add(roleId);
		return R.ok(sysPermissionService.findPermissionByRoleId(roleIds).stream().map(SysPermission::getPermissionId)
				.collect(Collectors.toList()));
	}

	/**
	 * 通过ID查询权限的详细信息
	 * @param id 权限ID
	 * @return 权限详细信息
	 */
	@GetMapping("/{id}")
	public R getById(@PathVariable String id) {
		return R.ok(sysPermissionService.getById(id));
	}

	//==================================================================================================================

	/**
	 * 新增权限
	 * @param sysPermission 权限信息
	 * @return 含ID 权限信息
	 */
	@SysLog("新增权限")
	@PostMapping
	@PreAuthorize("@pms.hasPermission('sys_permission_add')")
	public R add(@Valid @RequestBody SysPermission sysPermission) {
System.out.println(String.format("新增权限, id=%s name=%s permission=%s ", sysPermission.getPermissionId(), sysPermission.getName(), sysPermission.getPermission()));
		sysPermissionService.save(sysPermission);
		return R.ok(sysPermission);
	}

	/**
	 * 删除权限
	 * @param id 权限ID
	 * @return success/false
	 */
	@SysLog("删除权限")
	@DeleteMapping("/{id}")
	@PreAuthorize("@pms.hasPermission('sys_permission_del')")
	public R removeById(@PathVariable String id) {
System.out.println(String.format("删除权限, id=%s", id));
		return R.ok(sysPermissionService.removePermissionById(id));
	}

	/**
	 * 更新权限
	 * @param sysPermission
	 * @return
	 */
	@SysLog("更新权限")
	@PutMapping
	@PreAuthorize("@pms.hasPermission('sys_permission_edit')")
	public R update(@Valid @RequestBody SysPermission sysPermission) {
System.out.println(String.format("修改权限, id=%s name=%s permission=%s ", sysPermission.getPermissionId(), sysPermission.getName(), sysPermission.getPermission()));
		return R.ok(sysPermissionService.updatePermissionById(sysPermission));
	}

}
