package digital.laboratory.platform.sys.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.cloud.commons.lang.StringUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.sys.entity.Drug;
import digital.laboratory.platform.sys.entity.DrugType;
import digital.laboratory.platform.sys.mapper.DrugMapper;
import digital.laboratory.platform.sys.service.DrugService;
import digital.laboratory.platform.sys.service.DrugTypeService;
import digital.laboratory.platform.sys.vo.SuspiciousDrugVO;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 毒品清单服务实现类
 *
 * @author Zhang Xiaolong created at  2022-06-23
 * @describe 毒品清单服务实现类
 */
@Service
public class DrugServiceImpl extends ServiceImpl<DrugMapper, Drug> implements DrugService {
    @Resource
    private DrugTypeService drugTypeService;
    @Override
    public int getDrugCountByRootId(String typeId) {
       //计算出这一类的毒品清单数量
       String rootId=drugTypeService.getRootID(typeId);
       List<String>  typeIds=drugTypeService.getAllNodeByRootId(rootId);
       int count= this.list(Wrappers.<Drug>lambdaQuery()
               .in(Drug::getDrugType,typeIds)).size();
        return count;
    }

    /**
     * 获取所有顶层毒品类型的数据，以map的列表返回
     * @param drugType
     * @param name
     * @param englishName
     * @param casCode
     * @return
     */
    @Override
    public  List<SuspiciousDrugVO> drugMapList(String drugType, String name, String englishName, String casCode) {
        List<DrugType> drugTypeList = drugTypeService.list(); // 获取所有的毒品类型
        List<DrugType> rootTypeList = drugTypeList.stream().filter(dType -> dType.getPid().equals("0")).collect(Collectors.toList()); // 筛选出顶层
        Map<String, List<String>> drugTypeHierarchy = drugTypeService.buildDrugTypeHierarchy(drugTypeList); // 构建毒品类型的父子映射关系
        // 最后封装的结果
        boolean notBlankName = StrUtil.isNotBlank(name);
        boolean notBlankEnName = StrUtil.isNotBlank(englishName);
        boolean notBlankCasCode = StrUtil.isNotBlank(casCode);
        List<SuspiciousDrugVO> resultList = new ArrayList<>();

        for (DrugType type : rootTypeList) {
            List<String> descendantTypeIds = getDescendantTaskIds(type.getId(), drugTypeHierarchy);
            descendantTypeIds.add(type.getId());
            SuspiciousDrugVO suspiciousDrugVO = new SuspiciousDrugVO();
            suspiciousDrugVO.setDrugType(type.getName());
            suspiciousDrugVO.setDrugList(super.list(Wrappers.<Drug>lambdaQuery()
                    .in(Drug::getDrugType, descendantTypeIds)
                    .and(
                            StrUtil.isNotBlank(drugType)
                                    && drugType.equals(type.getName())
                                    && (notBlankName || notBlankEnName || notBlankCasCode) ,
                            wrapper ->
                                    wrapper
                                            .like(notBlankName, Drug::getName, name)
                                            .or()
                                            .like(notBlankEnName, Drug::getEnglishName, englishName)
                                            .or()
                                            .like(notBlankCasCode, Drug::getCasCode, casCode)
                    )
                    .orderByAsc(Drug::getOrderIndex)
            ));
            if (StrUtil.isNotBlank(drugType) && drugType.equals(type.getName())) {
                if (notBlankName) {
                    suspiciousDrugVO.setKeyword(name);
                    suspiciousDrugVO.setKeywordType("name");
                } else if (notBlankEnName) {
                    suspiciousDrugVO.setKeyword(englishName);
                    suspiciousDrugVO.setKeywordType("englishName");
                } else if (notBlankCasCode) {
                    suspiciousDrugVO.setKeyword(casCode);
                    suspiciousDrugVO.setKeywordType("casCode");
                }
            }
            resultList.add(suspiciousDrugVO);
        }

        return resultList;
    }

    /**
     * 分页查询
     * @param page
     * @param drug
     * @return
     */
    @Override
    public IPage<Drug> getDrugPage(Page page, Drug drug) {
        Map<String, List<String>> drugTypeHierarchy = drugTypeService.buildDrugTypeHierarchy(drugTypeService.list()); // 构建毒品类型的父子映射关系
        List<String> descendantTaskIds = null;
        if (StringUtils.isNotBlank(drug.getDrugType())) {
            descendantTaskIds = getDescendantTaskIds(drug.getDrugType(), drugTypeHierarchy);
            descendantTaskIds.add(drug.getDrugType());
        }
        QueryWrapper queryWrapper = Wrappers.<Drug>query()
                .in(CollUtil.isNotEmpty(descendantTaskIds),"drug_type", descendantTaskIds)
                .like(StringUtils.isNotBlank(drug.getName()),"name",drug.getName())
                .like(StringUtils.isNotBlank(drug.getCasCode()),"cas_code",drug.getCasCode())
                .like(StringUtils.isNotBlank(drug.getEnglishName()),"english_name",drug.getEnglishName())
                .eq(StringUtils.isNotBlank(drug.getCode()),"english_name",drug.getCode());
        return this.page(page,queryWrapper);
    }

    /**
     * 获取指定类型的所有后裔类型 ID
     */
    private List<String> getDescendantTaskIds(String parentId, Map<String, List<String>> taskChildrenMap) {
        List<String> result = new ArrayList<>();
        List<String> children = taskChildrenMap.get(parentId);
        if (children != null && !children.isEmpty()) {
            result.addAll(children);  // 将当前子任务ID加入结果
            // 递归获取所有后裔任务ID
            for (String childId : children) {
                result.addAll(getDescendantTaskIds(childId, taskChildrenMap)); // 递归获取子任务的后裔
            }
        }
        return result;
    }
    @Override
    public Boolean addDrug(Drug drug) {
        if(!StringUtils.isNotBlank(drug.getName())){
            return false;
        }
        //判断是否有名字相同的，如果名字相同，抛出异常
        List<Drug> drugList=  this.list(Wrappers.<Drug>query().eq("name",drug.getName()));
        if(drugList.size()>0)
        {
            return false;
        }else {
            drug.setId(IdWorker.get32UUID().toUpperCase());
            drug.setCode(generateDrugCode(drug.getDrugType())+"");
            return  this.save(drug);
        }
    }
    /**
     * 生成毒品CODE
     * @return
     */
    public int generateDrugCode(String rootTypeId)
    {
        //求出这种类型包含了多少毒品，然后新的编号就是最大编号+1

        int count = getDrugCountByRootId(rootTypeId);
        return count;

    }
}
