package digital.laboratory.platform.sys.service.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.sys.entity.Message;
import digital.laboratory.platform.sys.entity.SysUser;
import digital.laboratory.platform.sys.mapper.MessageMapper;
import digital.laboratory.platform.sys.mapper.SysUserMapper;
import digital.laboratory.platform.sys.service.MessageService;
import digital.laboratory.platform.sys.service.SysUserQuery;
import digital.laboratory.platform.sys.vo.MessageVO;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * 消息服务实现类
 *
 * @author Zhang Xiaolong created at  2023-02-02
 * @describe 消息 服务实现类
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class MessageServiceImpl extends ServiceImpl<MessageMapper, Message> implements MessageService {
	private final SysUserQuery userService;
	private final SysUserMapper userMapper;

	/**
	 * 根据 id 取指定消息
	 * 如果指定 ownerId, 则 ownerId 也作为查询条件
	 * @param messageId
	 * @param ownerId
	 * @return
	 */
	@Override
	public Message getMessageById(String messageId, String ownerId) {
		List<MessageVO> messageList = baseMapper.getMessageVOList(Wrappers.<Message>query()
				.eq("id", messageId)
				.eq(StrUtil.isNotBlank(ownerId), "owner_id", ownerId)
		);
		if ((messageList!=null) && (messageList.size()>0)) {
			return messageList.get(0);
		}
		else {
			return null;
		}
	}

	/**
	 * 分页查询消息
	 * @param page		分页参数
	 * @param text		查询内容
	 * @param isread	是否已读
	 * @param ownerId	拥有者id
	 * @return
	 */
	@Override
	public IPage<MessageVO> getMessagePage(Page<Message> page, String text, Boolean isread, String ownerId) {
		IPage<MessageVO> messageList = baseMapper.getMessageVOPage(page, Wrappers.<Message>query()
				.eq(isread!=null, "isread", isread)
				.like(StrUtil.isNotBlank(text), "content", text)
				.eq(StrUtil.isNotBlank(ownerId), "owner_id", ownerId)
				.orderByDesc("create_time")
		);
		return messageList;
	}

	/**
	 * 查询消息
	 * @param text		查询内容
	 * @param isread	是否已读
	 * @param ownerId	拥有者id
	 * @return
	 */
	@Override
	public List<MessageVO> getMessageList(String text, Boolean isread, String ownerId) {
		List<MessageVO> messageList = baseMapper.getMessageVOList(Wrappers.<Message>query()
				.eq(isread!=null, "isread", isread)
				.like(StrUtil.isNotBlank(text), "content", text)
				.eq(StrUtil.isNotBlank(ownerId), "owner_id", ownerId)
				.orderByDesc("create_time")
		);
		return messageList;
	}

	/**
	 * 发送消息给指定用户
	 * @param receiver
	 * @param text
	 * @param sender
	 * @return
	 */
	@Override
	public boolean sendMessageToUser(String receiver, String text, String sender) {
		Message msg = new Message();
		msg.setId(IdWorker.get32UUID().toUpperCase());
		msg.setOwnerId(receiver);
		msg.setSenderId(sender);
		msg.setContent(text);
		msg.setIsread(false);
		msg.setIsSystemMessage(false);

		return super.save(msg);
	}

	/**
	 * 发送消息给指定机构的拥有某些权限的用户
	 *
	 * @param orgId				指定机构
	 * @param includeSubOrg		是否包括子机构
	 * @param permissions		权限列表
	 * @param text				消息内容
	 * @param sender			发送者
	 * @return
	 */
		@Override
		public boolean sendMessage(String orgId, boolean includeSubOrg, List<String> permissions, String text, String sender) {

		List<SysUser> userList = null;
		if (StrUtil.isEmpty(orgId)) {
			// orgId 为空, 不限制机构
			userList = userMapper.listAllUsersByPermission(permissions);
		}
		else {
			// 限制为只在指定机构内的用户
			userList = userService.listUsersByPermission(orgId, includeSubOrg, permissions);
		}

		if (userList != null) {
			for (SysUser user : userList) {
				Message msg = new Message();
				msg.setId(IdWorker.get32UUID().toUpperCase());
				msg.setOwnerId(user.getUserId());
				msg.setSenderId(sender);
				msg.setContent(text);
				msg.setIsread(false);
				msg.setIsSystemMessage(false);
				super.save(msg);
			}
			return true;
		}
		return false;
	}


}
