package digital.laboratory.platform.sys.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.lang.tree.Tree;
import cn.hutool.core.lang.tree.TreeNode;
import cn.hutool.core.lang.tree.TreeUtil;
import cn.hutool.core.util.StrUtil;
import digital.laboratory.platform.sys.entity.SysPermission;
import digital.laboratory.platform.sys.entity.SysRolePermission;
import digital.laboratory.platform.sys.mapper.SysPermissionMapper;
import digital.laboratory.platform.sys.mapper.SysRolePermissionMapper;
import digital.laboratory.platform.sys.service.SysPermissionService;
import digital.laboratory.platform.common.core.constant.CacheConstants;
import digital.laboratory.platform.common.core.constant.CommonConstants;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.RequiredArgsConstructor;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import javax.validation.constraints.NotNull;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * <p>
 * 权限权限表 服务实现类
 * </p>
 *
 * @author lengleng
 * @since 2017-10-29
 */
@Service
@RequiredArgsConstructor
public class SysPermissionServiceImpl extends ServiceImpl<SysPermissionMapper, SysPermission> implements SysPermissionService {

	private final SysRolePermissionMapper sysRolePermissionMapper;

	@Override
	@Cacheable(value = CacheConstants.PERMISSION_DETAILS, key = "#roleIds  + '_permission'", unless = "#result == null")
	public Set<SysPermission> findPermissionByRoleId(List<String> roleIds) {
		if ((roleIds == null) || (roleIds.size() == 0)) {
			return new HashSet<SysPermission>();        // 如果参数是空的, 则返回一个空的集合
		}
		return baseMapper.listPermissionsByRoleId(roleIds);
	}

	/**
	 * 级联删除权限
	 * @param id 权限ID
	 * @return true成功, false失败
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	@CacheEvict(value = CacheConstants.PERMISSION_DETAILS, allEntries = true)
	public Boolean removePermissionById(String id) {
		// 查询父节点为当前节点的节点
		List<SysPermission> permissionList = this.list(Wrappers.<SysPermission>query().lambda().eq(SysPermission::getParentId, id));

		Assert.isTrue(CollUtil.isEmpty(permissionList), "权限含有下级不能删除");

		sysRolePermissionMapper.delete(Wrappers.<SysRolePermission>query().lambda().eq(SysRolePermission::getPermissionId, id));
		// 删除当前权限及其子权限
		return this.removeById(id);
	}

	@Override
	@CacheEvict(value = CacheConstants.PERMISSION_DETAILS, allEntries = true)
	public Boolean updatePermissionById(SysPermission sysPermission) {
		return this.updateById(sysPermission);
	}

	/**
	 * 构建树查询 1. 不是懒加载情况, 查询全部 2. 是懒加载, 根据parentId 查询 2.1 父节点为空, 则查询ID -1
	 * @param lazy 是否是懒加载
	 * @param parentId 父节点ID
	 * @return
	 */
	@Override
	public List<Tree<String>> treePermission(boolean lazy, String parentId) {
		if (!lazy) {
			List<TreeNode<String>> collect = baseMapper
					.selectList(Wrappers.<SysPermission>lambdaQuery().orderByAsc(SysPermission::getSort)).stream()
					.map(getNodeFunction()).collect(Collectors.toList());

			return TreeUtil.build(collect, CommonConstants.PERMISSION_TREE_ROOT_ID);
		}

		String parent = parentId == null ? CommonConstants.PERMISSION_TREE_ROOT_ID : parentId;

		List<TreeNode<String>> collect = baseMapper
				.selectList(
						Wrappers.<SysPermission>lambdaQuery().eq(SysPermission::getParentId, parent).orderByAsc(SysPermission::getSort))
				.stream().map(getNodeFunction()).collect(Collectors.toList());

		return TreeUtil.build(collect, parent);
	}

	/**
	 * 查询权限
	 * @param all 全部权限
	 * @param parentId 父节点ID
	 * @return
	 */
	@Override
	public List<Tree<String>> filterPermission(Set<SysPermission> all, String parentId) {
		List<TreeNode<String>> collect = all.stream()
//				.filter(permission -> PermissionTypeEnum.BACK_SIDE.getType().equals(permission.getType()))
				.filter(permission -> StrUtil.isNotBlank(permission.getPath())).map(getNodeFunction()).collect(Collectors.toList());
		String parent = parentId == null ? CommonConstants.PERMISSION_TREE_ROOT_ID : parentId;
		return TreeUtil.build(collect, parent);
	}

	@NotNull
	private Function<SysPermission, TreeNode<String>> getNodeFunction() {
		return permission -> {
			TreeNode<String> node = new TreeNode<>();
			node.setId(permission.getPermissionId());
			node.setParentId(permission.getParentId());
			node.setName(permission.getName());
			node.setWeight(permission.getSort());
//System.out.println(String.format("getNodeFunction: %s %s", permission.getPermissionId(), permission.getName()));
			// 扩展属性
			Map<String, Object> extra = new HashMap<>();
			extra.put("permission", permission.getPermission());
			extra.put("sort", permission.getSort());
			extra.put("type", permission.getType());
			extra.put("comments", permission.getComments());

			extra.put("path", permission.getPath());
			extra.put("icon", permission.getIcon());
			extra.put("keepAlive", permission.getKeepAlive());
			extra.put("component", permission.getComponent());
			extra.put("label", permission.getName());

			extra.put("createTime", permission.getCreateTime());
			extra.put("updateTime", permission.getUpdateTime());
			extra.put("createBy", permission.getCreateBy());
			extra.put("updateBy", permission.getUpdateBy());

			node.setExtra(extra);
			return node;
		};
	}

}
